@@ -2,10 +2,29 @@
 # -*- coding: utf-8 -*-
 
 import cv2
+import time
 import numpy as np
-from .error import TemplateInputError
 from PIL import Image
 
+from airtest.utils.logger import get_logger
+from .error import TemplateInputError
+
+LOGGING = get_logger(__name__)
+
+
+def print_run_time(func):
+
+    def wrapper(self, *args, **kwargs):
+        start_time = time.time()
+        ret = func(self, *args, **kwargs)
+        t = time.time() - start_time
+        LOGGING.debug("%s() run time is %.2f s." % (func.__name__, t))
+        if ret and isinstance(ret, dict):
+            ret["time"] = t
+        return ret
+
+    return wrapper
+
 
 def generate_result(middle_point, pypts, confi):
     """Format the result: 定义图像识别结果格式."""
@@ -48,7 +67,7 @@ def img_2_string(img):
 
 
 def string_2_img(pngstr):
-    nparr = np.fromstring(pngstr, np.uint8)
+    nparr = np.frombuffer(pngstr, np.uint8)
     img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
     return img
 
@@ -67,3 +86,22 @@ def cv2_2_pil(cv2_image):
     cv2_im = cv2.cvtColor(cv2_image, cv2.COLOR_BGR2RGB)
     pil_im = Image.fromarray(cv2_im)
     return pil_im
+
+
+def compress_image(pil_img, path, quality, max_size=None):
+    """
+    Save the picture and compress
+
+    :param pil_img: PIL image
+    :param path: save path
+    :param quality: the image quality, integer in range [1, 99]
+    :param max_size: the maximum size of the picture, e.g 1200
+    :return:
+    """
+    if max_size:
+        # The picture will be saved in a size <= max_size*max_size
+        pil_img.thumbnail((max_size, max_size), Image.ANTIALIAS)
+    quality = int(round(quality))
+    if quality <= 0 or quality >= 100:
+        raise Exception("SNAPSHOT_QUALITY (" + str(quality) + ") should be an integer in the range [1,99]")
+    pil_img.save(path, quality=quality, optimize=True)
