@@ -1,6 +1,7 @@
 # _*_ coding:UTF-8 _*_
 import os
 import sys
+import stat
 import threading
 from functools import wraps
 from six import string_types
@@ -63,6 +64,23 @@ def _cleanup():
         func(*args, **kwargs)
 
 
+def kill_proc(proc):
+    """
+    Kill the process and close _io.BufferedWriter to avoid `ResourceWarning: unclosed file <_io.BufferedWriter name=6>`
+
+    Args:
+        proc: subprocess.Popen()
+
+    Returns:
+
+    """
+    proc.kill()
+    # https://bugs.python.org/issue35182
+    # 部分低版本的python中，重复关闭io流可能会导致异常报错，因此需要额外加入判断closed
+    if proc.stdout and not proc.stdout.closed:
+        proc.communicate()
+
+
 # atexit.register(_cleanup)
 
 _shutdown = threading._shutdown
@@ -118,3 +136,18 @@ def ready_method(func):
             setattr(inst, key, True)
         return ret
     return wrapper
+
+
+def make_file_executable(file_path):
+    """
+    If the path does not have executable permissions, execute chmod +x
+    :param file_path:
+    :return:
+    """
+    if os.path.isfile(file_path):
+        mode = os.lstat(file_path)[stat.ST_MODE]
+        executable = True if mode & stat.S_IXUSR else False
+        if not executable:
+            os.chmod(file_path, mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
+        return True
+    return False
